function [d_optim,x,x_initial,points_p] = transport_problem_All_l2_VK(y,q,x_num,iprint,FileName)

%-------------- Input --------------
% y - set of positions of atoms in target (fixed) distribution: dimention x number_of_atoms 
% q - row-vector of probabilities of target distribution
% x_num - a number of atoms in approximated distribution
% iprint - 1/0 print or not rezults in text files
% FileName - name of txt file for solving log
%
%-------------- Output -------------
% d_optim - sequantial values of objective function (last value should be minimal)
% x - the best positions of atoms in approximated distribution
% x_initial - positions of atoms in approximated distribution after zero step - solving k-means clustering problem
% points_p - sequantial vectors of probabilities (last vector corresponds to the best solution) 
%-----------------------------------------


tic

fid=-1;
if iprint>0, fid=fopen(FileName,'w');
  if fid>0, i=fprintf(fid,'  Step                           Objective     Time_Load Time_prep Time_sol\n'); end
end
y_num=size(y,2);
if y_num < x_num, error('Dimension X must be less than dimension of Y') 
end

d_optim=[];
points_p=[];
step_max=100000;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Step 0
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

objective_00=1e100;
for i=1:100; [iddd,xxxx,summm,DDDD] = kmeans(y',x_num,'MaxIter',100000);    % 'distance'='sqEuclidean' by defaul

  w=sum(summm)/y_num; 

%cv1  w=0; for ii=1:y_num; j=iddd(ii); w=w+sqrt(DDDD(ii,j))*q(j); end     % this variant more coresponds to our criteria but final rezult is worse !!!
  
  if w<objective_00, objective_00=w; 
    idx3=iddd; x_initial=xxxx'; D=DDDD;
  end 
end
%[idx3,x_initial,sumd,D] = kmeans(y',x_num,'MaxIter',100000);    % 'distance'='sqEuclidean' by defaul
%x_initial=x_initial';
%objective_00=sum(sumd)/y_num;

point_p_data_resul=zeros(x_num,1);
objective_0=0;
for i=1:y_num; j=idx3(i); point_p_data_resul(j)=point_p_data_resul(j)+q(j); 
  objective_0=objective_0+sqrt(D(i,j))*q(j);
end
if fid>0, i=fprintf(fid, 'Objective_0               %17.10g\n',objective_0); end
%}


points_p=[points_p,point_p_data_resul];
d_optim = [d_optim,objective_0];

x=x_initial;
%==========================================================================

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                Cycle
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% INPUT: y,q,x,fid       OUTPUT: points_p

key_stop = 0; step_count=0;
objective_1=1e300; 
objective_2=1e300;

%d_optim=[];

while key_stop ~= 1 && step_count<step_max
  step_count=step_count+1;

  
  
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Step 2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   objective_2p=objective_2;

  if key_stop==0 
    
    [objective_2,left_w_j] = transport_problem_2_l2_VK(y,q,x,fid);
    
    d_optim = [d_optim,objective_2];
   
     %%%%% Calculate p_i as sum of w_i_j
     points_p = [points_p,sum(left_w_j')'];
  end %if key_stop==0

  if objective_2>=objective_2p && key_stop == 0, key_stop = 1; 
    disp(sprintf('No decreasing of objective for second subproblem on step # = %g',step_count))
  end
      
  
 
  
  
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Step 1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  objective_1p=objective_1;   
  if key_stop==0 
    %[objective_1,x_new] = transport_problem_1_l2_VK(y,left_w_j,x,fid);
    [objective_1,x_new] = transport_problem_Sqrt_quadratic_VK(y,left_w_j,x,fid);
    d_optim = [d_optim,objective_1];
  end
  
  if length(d_optim)>1
    if d_optim(end-1) - d_optim(end)<=10^(-7) || objective_1>=objective_1p 
        disp(' '); disp(sprintf('No changes of objective for step # = %g',step_count))
        d_optim(end)=[];
        key_stop = 1;
    else
        x = x_new;
    end    
  end

end %while key_stop ~= 1 && step_count<10
%-------------------- END CYCLE ---------------------

disp(' '); toc
disp(' '); 
if iprint>0, i=fprintf(fid, 'Prepare_Exit                                  %7.2f\n',toc); end

if fid>0, fclose(fid); end

end % function transport_problem_All_l2

%==========================================================================
